/**
 * Merges custom RPC options with defaults. Returns a new instance and keeps
 * the "defaults" and the "options" unmodified.
 *
 * Merges `RpcMetadata` "meta", overwriting values from "defaults" with
 * values from "options". Does not append values to existing entries.
 *
 * Merges "typeRegistry" by creating a new array that contains types from
 * "options" first, then types from "defaults".
 *
 * Merges "interceptors" by creating a new array that contains interceptors
 * from "defaults" first, then interceptors from "options".
 *
 * Works with objects that extend `RpcOptions`, but only if the added
 * properties are of type Date, primitive like string, boolean, or Array
 * of primitives. If you have other property types, you have to merge them
 * yourself.
 */
export function mergeExtendedRpcOptions(defaults, options) {
    if (!options)
        return defaults;
    let o = Object.assign({ meta: {}, typeRegistry: [], interceptors: [] }, defaults);
    for (let k of Object.keys(options)) {
        let v = options[k];
        if (v === undefined)
            break;
        switch (k) {
            case "meta":
                o.meta = {};
                if (defaults.meta)
                    for (let [mk, mv] of Object.entries(defaults.meta))
                        o.meta[mk] = Array.isArray(mv) ? mv.concat() : mv;
                for (let [mk, mv] of Object.entries(v))
                    o.meta[mk] = Array.isArray(mv) ? mv.concat() : mv;
                break;
            case "typeRegistry":
                o.typeRegistry = defaults.typeRegistry ? [...v, ...defaults.typeRegistry] : v.concat();
                break;
            case "interceptors":
                o.interceptors = defaults.interceptors ? defaults.interceptors.concat(v) : v.concat();
                break;
            default:
                if (v instanceof Date)
                    o[k] = new Date(v.getTime());
                else if (Array.isArray(v))
                    o[k] = v.concat();
                else
                    o[k] = v;
                break;
        }
    }
    return o;
}
//# sourceMappingURL=rpc-options.js.map